#define FIGURES_BRO_IMG // Loads bro img
#include "bro.h"
#include "bullet.h"
#include "enemy.h"

#include "../lib/MonochromeLib.h"
#include "../lib/coll.h"
#include "../lib/ECode.h"

#include "../scenes/levels/level.h"


static C_AABB _me;
static char _moving = 0;
static char _init = 0;
static float _hp = 100;
static char _dead = 0;
static Mvt _mvt = DOWN;

static void bro_init(void)
{
	_me.x = 56;
	_me.y = 24;
	_me.w = _me.h = 16;
	_init = 1;
}

C_AABB* bro_get()
{
	if(_init == 0)
		bro_init();

	return &_me;
}

float bro_get_hp(void)
{
	return _hp;
}

int bro_set_hp(float damage)
{
	_hp -= damage;

	if(_hp <= 0 && _dead != 1) {
		_hp = 0;
		_dead = 1;
		_me.y += 7;
		_me.h = 9;
		return 1;
	}

	return 0;
}

int bro_is_dead(void)
{
	return (_dead > 0) ? 1 : 0;
}

void bro_move(int x, int y)
{
	int i ,j, oldx = _me.x, oldy = _me.y;
	Enemy *e;

	if(_init == 0)
		bro_init();

	if(_hp > 0)
	{
		// Bro' collision

		_me.x += x;
		_me.y += y;

		if(level_collide(&_me) == 1) {
			_me.x -= x;
			_me.y -= y;

			for(i = 0; i < ABS(x); i++)
			{
				_me.x += SGN(x);
				if(level_collide(&_me) == 1) {
					_me.x -= SGN(x);
					break;
				}
			}

			for(j = 0; j < ABS(y); j++)
			{
				_me.y += SGN(y);
				if(level_collide(&_me) == 1) {
					_me.y -= SGN(y);
					break;
				}
			}
		}


		e = enemy_collide(&_me);

		if(e != NULL && e->alive != 0) {
			_me.x -= x;
			_me.y -= y;

			for(i = 0; i < ABS(x); i++)
			{
				_me.x += SGN(x);
				e = enemy_collide(&_me);
				if(e != NULL) {
					_me.x -= SGN(x);
					break;
				}
			}

			for(j = 0; j < ABS(y); j++)
			{
				_me.y += SGN(y);
				e = enemy_collide(&_me);
				if(e != NULL) {
					_me.y -= SGN(y);
					break;
				}
			}
		}

		// Movment determination

		if(x > 0)
			_mvt = RIGHT;
		if(x < 0)
			_mvt = LEFT;
		if(y > 0)
			_mvt = DOWN;
		if(y < 0)
			_mvt = UP;

		// Is moving ?

		if(x != 0 || y != 0)
			_moving = 1;
		else
			_moving = 0;

		if(oldx == _me.x && oldy == _me.y)
			_moving = 0;
	}
}


void bro_render(void)
{
	static int count = 0;
	C_AABB* c;

	if(_init == 0)
		bro_init();

	c = level_get_camera();

	// Rendering

	if(_hp > 0)
	{
		if(_moving == 1)
		{
			switch(_mvt) {
				case LEFT:

					ML_bmp_or_cl(bro_weapon_left, _me.x-c->x-1, _me.y-c->y+7, 8, 4);
					if(count <= 64) {
						ML_bmp_16_or_cl(bro_side1_left, _me.x-c->x, _me.y-c->y);
					} else if ((count > 64 && count <= 128) || (count > 192 && count <= 256)) {
						ML_bmp_16_or_cl(bro_sideIdle_left, _me.x-c->x, _me.y-c->y);
					} else {
						ML_bmp_16_or_cl(bro_side2_left, _me.x-c->x, _me.y-c->y);
					}

					break;
				case RIGHT:

					ML_bmp_or_cl(bro_weapon_right, _me.x-c->x+9, _me.y-c->y+7, 8, 4);
					if(count <= 64) {
						ML_bmp_16_or_cl(bro_side1_right, _me.x-c->x, _me.y-c->y);
					} else if ((count > 64 && count <= 128) || (count > 192 && count <= 256)) {
						ML_bmp_16_or_cl(bro_sideIdle_right, _me.x-c->x, _me.y-c->y);
					} else {
						ML_bmp_16_or_cl(bro_side2_right, _me.x-c->x, _me.y-c->y);
					}

					break;

				case UP:

					ML_bmp_or_cl(bro_weapon_up, _me.x-c->x+12, _me.y-c->y+5, 4, 8);
					if(count <= 64) {
						ML_bmp_16_or_cl(bro_back1_right, _me.x-c->x, _me.y-c->y);
					} else if ((count > 64 && count <= 128) || (count > 192 && count <= 256)) {
						ML_bmp_16_or_cl(bro_backIdle_right, _me.x-c->x, _me.y-c->y);
					} else {
						ML_bmp_16_or_cl(bro_back2_right, _me.x-c->x, _me.y-c->y);
					}

					break;
				case DOWN:

					ML_bmp_or_cl(bro_weapon_down, _me.x-c->x, _me.y-c->y+8, 4, 8);
					if(count <= 64) {
						ML_bmp_16_or_cl(bro_face1_right, _me.x-c->x, _me.y-c->y);
					} else if ((count > 64 && count <= 128) || (count > 192 && count <= 256)) {
						ML_bmp_16_or_cl(bro_faceIdle_right, _me.x-c->x, _me.y-c->y);
					} else {
						ML_bmp_16_or_cl(bro_face2_right, _me.x-c->x, _me.y-c->y);
					}

					break;
			}
				
			count += 20;
			count %= 256;
		} else {
			switch(_mvt) {
				case LEFT:

					ML_bmp_or_cl(bro_weapon_left, _me.x-c->x-1, _me.y-c->y+7, 8, 4);
					ML_bmp_16_or_cl(bro_sideIdle_left, _me.x-c->x, _me.y-c->y);

					break;
				case RIGHT:

					ML_bmp_or_cl(bro_weapon_right, _me.x-c->x+9, _me.y-c->y+7, 8, 4);
					ML_bmp_16_or_cl(bro_sideIdle_right, _me.x-c->x, _me.y-c->y);				

					break;

				case UP:

					ML_bmp_or_cl(bro_weapon_up, _me.x-c->x+12, _me.y-c->y+5, 4, 8);
					ML_bmp_16_or_cl(bro_backIdle_right, _me.x-c->x, _me.y-c->y);				

					break;
				case DOWN:

					ML_bmp_or_cl(bro_weapon_down, _me.x-c->x, _me.y-c->y+8, 4, 8);
					ML_bmp_16_or_cl(bro_faceIdle_right, _me.x-c->x, _me.y-c->y);				

					break;
			}
		}
		
		_moving = 0;
	} else {
		switch(_mvt) {
			case LEFT:
			case UP:
				ML_bmp_or_cl(bro_weapon_up, _me.x-c->x+12, _me.y-c->y+5, 4, 8);
				ML_bmp_or_cl(bro_dead_left, _me.x-c->x, _me.y-c->y, _me.w, _me.h);

				break;
			case RIGHT:
			case DOWN:
				ML_bmp_or_cl(bro_weapon_up, _me.x-c->x+12, _me.y-c->y+5, 4, 8);
				ML_bmp_or_cl(bro_dead_right, _me.x-c->x, _me.y-c->y, _me.w, _me.h);
				break;
		}
	}
	
}

void bro_fire(void)
{
	static int last = 0;
	int x, y;
	C_AABB* c;

	if(_hp > 0)
	{
		c = level_get_camera();

		if(last == 0)
		{
			switch(_mvt) {
				case LEFT:
					x = _me.x-3;
					y = _me.y+8;
					break;
				case RIGHT:
					x = _me.x+17;
					y = _me.y+8;
					break;
				case UP:
					x = _me.x+13;
					y = _me.y+4;
					break;
				case DOWN:
					x = _me.x+1;
					y = _me.y+15;
					break;
			}
			bullet_add(_mvt, x, y, 7);
			last = 2;
		} else {
			last--;
		}
	}
}