#define FIGURES_BULLET_IMG
#include "bullet.h"
#include "enemy.h"
#include "bro.h"
#include "../lib/coll.h"
#include "../lib/ECode.h"
#include "../lib/MonochromeLib.h"
#include "../scenes/levels/level.h"

static BulletStack _stack;
static char _init = 0;

static void bullet_init(void)
{
	_stack.first = NULL;
	_init = 1;
}

void bullet_add(Mvt mvt, int x, int y, int vel)
{
	Bullet *new, *ptr;

	if(_init == 0)
		bullet_init();

	new = calloc(1, sizeof(Bullet));

	new->box.x = x - 1;
	new->box.y = y - 1;
	new->box.w = 3;
	new->box.h = 3;

	new->vel = vel;
	new->dir = mvt;

	new->next = NULL;

	ptr = _stack.first;

	if(ptr == NULL) {
		_stack.first = new;
	} else {
		while(ptr->next)
			ptr = ptr->next;

		ptr->next = new;
	}
}

void bullet_free(Bullet *blt)
{
	Bullet *ptr;

	if(_init == 0)
		bullet_init();

	ptr = _stack.first;
	while(ptr)
	{
		if(ptr->next == blt) {
			ptr->next = blt->next;
			free(blt);
			return;
		}

		ptr = ptr->next;
	}
}

void bullet_reset(void)
{
	Bullet *ptr, *tmp;
	ptr = _stack.first;

	while(ptr)
	{
		tmp = ptr->next;
		free(ptr);
		ptr = tmp;
	}
}


static int bullet_compute(Bullet *blt)
{
	int w, h, r = 1;
	Enemy *e;

	if(_init == 0)
		bullet_init();

	w = level_get_width();
	h = level_get_height();

	switch(blt->dir) {
		case LEFT:
			blt->box.x -= blt->vel;
			break;
		case RIGHT:
			blt->box.x += blt->vel;
			break;
		case UP:
			blt->box.y -= blt->vel;
			break;
		case DOWN:
			blt->box.y += blt->vel;
			break;
	}

	if(level_collide(&(blt->box)) == 1) {
		r = 0;
	}
	
	e = enemy_collide(&(blt->box));

	if(e != NULL && e->alive != 0) {
		enemy_kill(e);
		r = 0;
	}

	return r;
}

void bullet_render(void)
{
	Bullet *blt, *next;
	C_AABB* c;

	if(_init == 0)
		bullet_init();

	next = _stack.first;

	c = level_get_camera();

	while(next)
	{
		blt = next;
		if(bullet_compute(blt) == 1)
		{
			ML_rectangle(
				blt->box.x - c->x,
				blt->box.y - c->y,
				blt->box.x + blt->box.w - c->x,
				blt->box.y + blt->box.h - c->y,
				0, ML_BLACK, ML_BLACK);
			next = blt->next;
		} else {
			next = blt->next;
			bullet_free(blt);
		}
	}
}