#define FIGURES_ENEMY_IMG // Loads enemy img
#include "enemy.h"
#include "bullet.h"

#include "../lib/MonochromeLib.h"
#include "../lib/coll.h"
#include "../lib/time.h"
#include "../lib/ECode.h"

#include "../scenes/levels/level.h"


#define SGN(x) (((x)==0)?(0):(((x)<0)?(-1):(1)))

static EnemyStack _stack;
static char _init = 0;


static void enemy_init(void)
{
	_stack.first = NULL;
	_init = 1;
}

void enemy_add(int x, int y)
{
	Enemy *new, *ptr;

	if(_init == 0)
		enemy_init();

	new = calloc(1, sizeof(Enemy));

	new->box.x = x;
	new->box.y = y;
	new->box.w = 16;
	new->box.h = 16;

	new->dir = UP;
	new->next = NULL;

	new->alive = 1;

	new->count = 0;

	ptr = _stack.first;

	if(ptr == NULL) {
		_stack.first = new;
	} else {
		while(ptr->next)
			ptr = ptr->next;

		ptr->next = new;
	}
}

void enemy_kill(Enemy *e)
{
	Enemy *ptr;
	C_AABB new;
	int w, h;
	w = level_get_width();
	h = level_get_height();

	if(_init == 0)
		enemy_init();

	ptr = _stack.first;
	while(ptr)
	{
		if(ptr == e && e->alive != 0) {
			e->alive = 0;
			e->box.y -= 7;
			e->box.h = 9;

			return;
		}

		ptr = ptr->next;
	}
}

void enemy_reset(void)
{
	Enemy *ptr, *tmp;
	ptr = _stack.first;

	while(ptr)
	{
		tmp = ptr->next;
		free(ptr);
		ptr = tmp;
	}
}


static Enemy* enemy_collide_exclude(C_AABB *box, Enemy *me)
{
	Enemy *e, *next;

	if(_init == 0)
		enemy_init();

	next = _stack.first;

	while(next)
	{
		e = next;

		if(e != me)
			if(C_TwoAABB(&(e->box), box) == TRUE && e->alive != 0)
				return e;

		next = e->next;
	}

	return (Enemy*)NULL;
}


void enemy_move(Enemy *e, int x, int y)
{
	int i ,j, oldx = e->box.x, oldy = e->box.y;
	C_AABB *bro;
	Enemy *coll;

	if(_init == 0)
		enemy_init();

	e->box.x += x;
	e->box.y += y;

	coll = enemy_collide_exclude(&(e->box), e);

	if(level_collide(&(e->box)) == 1 || (coll != NULL && coll->alive != 0)) {
		e->box.x -= x;
		e->box.y -= y;

		for(i = 0; i < ABS(x); i++)
		{
			e->box.x += SGN(x);
			if(level_collide(&(e->box)) == 1 || enemy_collide_exclude(&(e->box), e) != NULL) {
				e->box.x -= SGN(x);
				break;
			}
		}

		for(j = 0; j < ABS(y); j++)
		{
			e->box.y += SGN(y);
			if(level_collide(&(e->box)) == 1 || enemy_collide_exclude(&(e->box), e) != NULL) {
				e->box.y -= SGN(y);
				break;
			}
		}
	}

	bro = bro_get();

	if(C_TwoAABB(&(e->box), bro) == TRUE) {
		e->box.x -= x;
		e->box.y -= y;

		for(i = 0; i < ABS(x); i++)
		{
			e->box.x += SGN(x);
			if(C_TwoAABB(&(e->box), bro) == TRUE) {
				e->box.x -= SGN(x);
				break;
			}
		}

		for(j = 0; j < ABS(y); j++)
		{
			e->box.y += SGN(y);
			if(C_TwoAABB(&(e->box), bro) == TRUE) {
				e->box.y -= SGN(y);
				break;
			}
		}
		bro_set_hp(1);
	}

	// Movment determination

	if(x > 0)
		e->dir = RIGHT;
	if(x < 0)
		e->dir = LEFT;
	if(y > 0)
		e->dir = DOWN;
	if(y < 0)
		e->dir = UP;

	// Is moving ?

	if(x != 0 || y != 0)
		e->moving = 1;
	else
		e->moving = 0;

	if(oldx == e->box.x && oldy == e->box.y)
		e->moving = 0;
}

void enemy_fire(Enemy *e)
{
	int x, y;

	{
		switch(e->dir) {
			case LEFT:
				x = e->box.x-3;
				y = e->box.y+8;
				break;
			case RIGHT:
				x = e->box.x+17;
				y = e->box.y+8;
				break;
			case UP:
				x = e->box.x+13;
				y = e->box.y+4;
				break;
			case DOWN:
				x = e->box.x+1;
				y = e->box.y+15;
				break;
		}
		bullet_add(e->dir, x, y, 5);
	}
}

static int enemy_compute(Enemy *e)
{
	// Put IA here

	int x, y;
	char mod = 1;
	C_AABB *bro;

	bro = bro_get();

	if(e->alive != 0) {
		x = bro->x - e->box.x;
		y = bro->y - e->box.y;

		if(e->chasing == 0) {
			if(x*x + y*y < 100*100) // If distance btw bro and e < 64 px
			{
				e->chasing = 1;
			}
		} else {
			enemy_move(e, SGN(x), SGN(y));
			/*if(randBtw(1, 100) > 95 && e->alive == 1) {
				enemy_fire(e);
			}*/
		}
	}



	// Check if a bullet hit the enemy
}

Enemy* enemy_collide(C_AABB *box)
{
	Enemy *next;

	if(_init == 0)
		enemy_init();

	next = _stack.first;

	while(next)
	{
		if(C_TwoAABB(&(next->box), box) == TRUE && next->alive != 0)
			return next;

		next = next->next;
	}

	return (Enemy*)NULL;
}


void enemy_render()
{
	Enemy *e, *next;
	C_AABB* c;

	if(_init == 0)
		enemy_init();

	next = _stack.first;

	c = level_get_camera();

	while(next)
	{
		e = next;
		enemy_compute(e);
		if(e->alive == 1)
		{
			if(e->moving == 1)
			{
				switch(e->dir) {
					case LEFT:

						//ML_bmp_or_cl(enemy_weapon_left, e->box.x-c->x-1, e->box.y-c->y+7, 8, 4);
						if(e->count <= 64) {
							ML_bmp_16_or_cl(enemy_side1_left, e->box.x-c->x, e->box.y-c->y);
						} else if ((e->count > 64 && e->count <= 128) || (e->count > 192 && e->count <= 256)) {
							ML_bmp_16_or_cl(enemy_sideIdle_left, e->box.x-c->x, e->box.y-c->y);
						} else {
							ML_bmp_16_or_cl(enemy_side2_left, e->box.x-c->x, e->box.y-c->y);
						}

						break;
					case RIGHT:

						//ML_bmp_or_cl(enemy_weapon_right, e->box.x-c->x+9, e->box.y-c->y+7, 8, 4);
						if(e->count <= 64) {
							ML_bmp_16_or_cl(enemy_side1_right, e->box.x-c->x, e->box.y-c->y);
						} else if ((e->count > 64 && e->count <= 128) || (e->count > 192 && e->count <= 256)) {
							ML_bmp_16_or_cl(enemy_sideIdle_right, e->box.x-c->x, e->box.y-c->y);
						} else {
							ML_bmp_16_or_cl(enemy_side2_right, e->box.x-c->x, e->box.y-c->y);
						}

						break;

					case UP:

						//ML_bmp_or_cl(enemy_weapon_up, e->box.x-c->x+12, e->box.y-c->y+5, 4, 8);
						if(e->count <= 64) {
							ML_bmp_16_or_cl(enemy_back1_right, e->box.x-c->x, e->box.y-c->y);
						} else if ((e->count > 64 && e->count <= 128) || (e->count > 192 && e->count <= 256)) {
							ML_bmp_16_or_cl(enemy_backIdle_right, e->box.x-c->x, e->box.y-c->y);
						} else {
							ML_bmp_16_or_cl(enemy_back2_right, e->box.x-c->x, e->box.y-c->y);
						}

						break;
					case DOWN:

						//ML_bmp_or_cl(enemy_weapon_down, e->box.x-c->x, e->box.y-c->y+8, 4, 8);
						if(e->count <= 64) {
							ML_bmp_16_or_cl(enemy_face1_right, e->box.x-c->x, e->box.y-c->y);
						} else if ((e->count > 64 && e->count <= 128) || (e->count > 192 && e->count <= 256)) {
							ML_bmp_16_or_cl(enemy_faceIdle_right, e->box.x-c->x, e->box.y-c->y);
						} else {
							ML_bmp_16_or_cl(enemy_face2_right, e->box.x-c->x, e->box.y-c->y);
						}

						break;
				}
					
				e->count += 20;
				e->count %= 256;
			} else {
				switch(e->dir) {
					case LEFT:

						//ML_bmp_or_cl(enemy_weapon_left, e->box.x-c->x-1, e->box.y-c->y+7, 8, 4);
						ML_bmp_16_or_cl(enemy_sideIdle_left, e->box.x-c->x, e->box.y-c->y);

						break;
					case RIGHT:

						//ML_bmp_or_cl(enemy_weapon_right, e->box.x-c->x+9, e->box.y-c->y+7, 8, 4);
						ML_bmp_16_or_cl(enemy_sideIdle_right, e->box.x-c->x, e->box.y-c->y);				

						break;

					case UP:

						//ML_bmp_or_cl(enemy_weapon_up, e->box.x-c->x+12, e->box.y-c->y+5, 4, 8);
						ML_bmp_16_or_cl(enemy_backIdle_right, e->box.x-c->x, e->box.y-c->y);				

						break;
					case DOWN:

						//ML_bmp_or_cl(enemy_weapon_down, e->box.x-c->x, e->box.y-c->y+8, 4, 8);
						ML_bmp_16_or_cl(enemy_faceIdle_right, e->box.x-c->x, e->box.y-c->y);				

						break;
				}
			}
		} else {
			switch(e->dir) {
				case LEFT:
				case UP:
					ML_bmp_or_cl(enemy_dead_left, e->box.x-c->x, e->box.y-c->y, e->box.w, e->box.h);
					break;
				case RIGHT:
				case DOWN:
					ML_bmp_or_cl(enemy_dead_right, e->box.x-c->x, e->box.y-c->y, e->box.w, e->box.h);
					break;
			}

		}
		
		e->moving = 0;
		next = e->next;
	}
}